"""Generated message classes for appoptimize version v1beta.

The App Optimize API provides developers and platform teams with tools to
monitor, analyze, and improve the performance and cost-efficiency of their
cloud applications.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'appoptimize'


class AppoptimizeProjectsLocationsGetRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class AppoptimizeProjectsLocationsListRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class AppoptimizeProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class AppoptimizeProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class AppoptimizeProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class AppoptimizeProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class AppoptimizeProjectsLocationsReportsCreateRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsReportsCreateRequest object.

  Fields:
    parent: Required. The parent Google Cloud project that will own the
      report. This value does not define the scope of the report data. See
      `Report.scope` for setting the data scope. Format:
      `projects/{project}/locations/{location}`.
    report: A Report resource to be passed as the request body.
    reportId: Required. The ID to use for this report. This ID must be unique
      within the parent project and should comply with RFC 1034 restrictions
      (letters, numbers, and hyphen, with the first character a letter, the
      last a letter or a number, and a 63 character maximum).
  """

  parent = _messages.StringField(1, required=True)
  report = _messages.MessageField('Report', 2)
  reportId = _messages.StringField(3)


class AppoptimizeProjectsLocationsReportsDeleteRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsReportsDeleteRequest object.

  Fields:
    allowMissing: Optional. If set to true, and the report is not found, the
      request will succeed but no action will be taken on the server.
    name: Required. The name of the report to delete. Format:
      `projects/{project}/locations/{location}/reports/{report_id}`.
  """

  allowMissing = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)


class AppoptimizeProjectsLocationsReportsGetRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsReportsGetRequest object.

  Fields:
    name: Required. The name of the report to retrieve. Format:
      `projects/{project}/locations/{location}/reports/{report_id}`.
  """

  name = _messages.StringField(1, required=True)


class AppoptimizeProjectsLocationsReportsListRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsReportsListRequest object.

  Fields:
    pageSize: Optional. The maximum number of reports to return. The service
      may return fewer than this value. If unspecified, the server will
      determine the number of results to return.
    pageToken: Optional. A page token, received from a previous `ListReports`
      call. Provide this to retrieve the subsequent page. When paginating, all
      other parameters provided to `ListReports` must match the call that
      provided the page token.
    parent: Required. The parent project whose reports are to be listed.
      Format: `projects/{project}/locations/{location}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AppoptimizeProjectsLocationsReportsReadRequest(_messages.Message):
  r"""A AppoptimizeProjectsLocationsReportsReadRequest object.

  Fields:
    name: Required. The resource name of the report to query. Format:
      `projects/{project}/locations/{location}/reports/{report_id}`.
    readReportRequest: A ReadReportRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  readReportRequest = _messages.MessageField('ReadReportRequest', 2)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Column(_messages.Message):
  r"""Describes a single column within `Columns`.

  Fields:
    columns: If the `type` of this column is `RECORD`, this sub-field
      describes the nested structure.
    mode: The mode of the column, indicating if it is nullable, required, or
      repeated. Possible values: * `NULLABLE`: The column allows NULL values.
      * `REQUIRED`: The column does not allow NULL values. * `REPEATED`: The
      column contains an array of values.
    name: The name of the column. This field: * Contains only letters (a-z,
      A-Z), numbers (0-9), or underscores (_); * Start with a letter or
      underscore; and * Has a maximum length is 128 characters.
    type: The data type of the column. Supported values include: * `STRING` *
      `INT64` * `FLOAT64` * `BOOLEAN` * `TIMESTAMP` * `RECORD` `RECORD`
      indicates that the field contains a nested schema, described in the
      `columns` property of this `Column`.
  """

  columns = _messages.MessageField('Column', 1, repeated=True)
  mode = _messages.StringField(2)
  name = _messages.StringField(3)
  type = _messages.StringField(4)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListReportsResponse(_messages.Message):
  r"""Response message for the `ListReports` method.

  Fields:
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is empty, there are no subsequent pages.
    reports: The list of reports.
  """

  nextPageToken = _messages.StringField(1)
  reports = _messages.MessageField('Report', 2, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents metadata related to the creation of a Report. This value is
  embedded in the Operation object returned by `CreateReport`.
  """



class ReadReportRequest(_messages.Message):
  r"""Request message for the `ReadReport` method.

  Fields:
    pageSize: Optional. The maximum number of rows to return. The service may
      return fewer than this value. If unspecified, at most 10,000 rows will
      be returned per page. The maximum allowed value is 25,000; values above
      25,000 are coerced to 25,000.
    pageToken: Optional. A page token, received from a previous `ReadReport`
      call, to retrieve the subsequent page of results. When `page_token` is
      specified, `job_reference` must also be provided from the previous
      response, and the `statement` field must not be set.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)


class ReadReportResponse(_messages.Message):
  r"""Response message for the `ReadReport` method.

  Fields:
    columns: The columns describing the structure of the data in the `rows`
      field.
    nextPageToken: A token that can be sent as `page_token` in a subsequent
      `ReadReport` request to retrieve the next page of results. If this field
      is empty, there are no further pages.
    rows: A list of rows, where each row represents a record from the report.
  """

  columns = _messages.MessageField('Column', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  rows = _messages.MessageField('extra_types.JsonArray', 3, repeated=True)


class Report(_messages.Message):
  r"""A configuration that defines the parameters for the data represented by
  a report.

  Fields:
    dimensions: Required. A list of dimensions to include in the report.
      Supported values: * `project` * `application` * `service_or_workload` *
      `resource` * `resource_type` * `location` * `product_display_name` *
      `sku` * `month` * `day` * `hour` To aggregate results by time, specify
      at least one time dimension (`month`, `day`, or `hour`). All time
      dimensions use Pacific Time, respect Daylight Saving Time (DST), and
      follow these ISO 8601 formats: * `month`: `YYYY-MM` (e.g., `2024-01`) *
      `day`: `YYYY-MM-DD` (e.g., `2024-01-10`) * `hour`: `YYYY-MM-DDTHH`
      (e.g., `2024-01-10T00`) If the time range filter does not align with the
      selected time dimension, the range is expanded to encompass the full
      period of the finest-grained time dimension. For example, if the filter
      is `2026-01-10` through `2026-01-12` and the `month` dimension is
      selected, the effective time range expands to include all of January
      (`2026-01-01` to `2026-02-01`).
    expireTime: Output only. Timestamp in UTC of when this report expires.
      Once the report expires, it will no longer be accessible and the
      report's underlying data will be deleted.
    filter: Optional. A Common Expression Language (CEL) expression used to
      filter the data for the report. Predicates may refer to any dimension.
      Filtering must conform to these constraints: * All string field
      predicates must use exact string matches. * Multiple predicates
      referring to the same string field must be joined using the logical OR
      operator ('||'). * All other predicates must be joined using the logical
      AND operator (`&&`). * A predicate on a time dimension (e.g., `day`)
      specifying the start time must use a greater-than-or-equal-to comparison
      (`>=`). * A predicate on a time dimension specifying the end time must
      use a less-than comparison (`<`). Examples: 1. Filter by a specific
      resource type: `"resource_type == 'compute.googleapis.com/Instance'"` 2.
      Filter data points that fall within a specific absolute time interval:
      `"hour >= timestamp('2024-01-01T00:00:00Z') && hour <
      timestamp('2024-02-01T00:00:00Z')"` 3. Filter data points that fall
      within the past 72 hours: `"hour >= now - duration('72h')"` 4. Combine
      string predicate with time interval predicate: `"(location == 'us-east1'
      || location == 'us-west1') && hour >= timestamp('2023-12-01T00:00:00Z')
      && hour < timestamp('2024-02-01T00:00:00Z')"` If the filter omits time
      dimensions (`month`, `day`, `hour`), the report defaults to a 7-day
      range ending at the previous Pacific Time midnight, with Daylight Saving
      Time (DST) applied. For example, if the current Pacific Time is
      `2026-01-05T12:00:00`, the default range is `2025-12-29T00:00:00` to
      `2026-01-05T00:00:00` Pacific time.
    metrics: Required. A list of metrics to include in the report. Supported
      values: * `cost` * `cpu_mean_utilization` * `cpu_usage_core_seconds` *
      `cpu_allocation_core_seconds` * `cpu_p95_utilization` *
      `memory_mean_utilization` * `memory_usage_byte_seconds` *
      `memory_allocation_byte_seconds` * `memory_p95_utilization`
    name: Identifier. The name of this report.
    scopes: Optional. The resource containers for which to fetch data. Default
      is the project specified in the report's parent. No more than one scope
      is supported.
  """

  dimensions = _messages.StringField(1, repeated=True)
  expireTime = _messages.StringField(2)
  filter = _messages.StringField(3)
  metrics = _messages.StringField(4, repeated=True)
  name = _messages.StringField(5)
  scopes = _messages.MessageField('Scope', 6, repeated=True)


class Scope(_messages.Message):
  r"""Specifies the report scope.

  Fields:
    application: Required. An App Hub Application to fetch data from. Format:
      `"projects/{project}/locations/{location}/applications/{application}"`.
    project: Required. A Google Cloud Platform project to fetch data from.
      Format: `"projects/{project}"`.
  """

  application = _messages.StringField(1)
  project = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
