"""Generated message classes for lustre version v1alpha.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'lustre'


class AccessRule(_messages.Message):
  r"""A single policy group with IP-based access rules for the Managed Lustre
  instance.

  Enums:
    AccessModeValueValuesEnum: Optional. The access mode for the access rule
      nodemap. Default is READ_WRITE.
    SquashModeValueValuesEnum: Required. Squash mode for the access rule.

  Fields:
    accessMode: Optional. The access mode for the access rule nodemap. Default
      is READ_WRITE.
    ipAddressRanges: Required. The IP address ranges to which to apply this
      access rule. Accepts non-overlapping CIDR ranges (e.g.,
      `192.168.1.0/24`) and IP addresses (e.g., `192.168.1.0`).
    mountableSubdirectories: Optional. The list of non-root directories that
      can be mounted from clients in this NID range subset. Currently, there
      can be only a single directory at most. If no directory is mentioned,
      then the root directory will be accessible.
    name: Required. The name of the access rule policy group. Must be 16
      characters or less and include only alphanumeric characters or '_'.
    squashGid: Optional. Squash GID for the access rule. If the squash mode
      for this rule is ROOT_SQUASH, root users matching the ip_address_ranges
      are squashed to this GID. Defaults to 0 (no root squash).
    squashMode: Required. Squash mode for the access rule.
    squashUid: Optional. Squash UID for the access rule. If the squash mode
      for this rule is ROOT_SQUASH, root users matching the ip_address_ranges
      are squashed to this UID. Defaults to 0 (no root squash).
  """

  class AccessModeValueValuesEnum(_messages.Enum):
    r"""Optional. The access mode for the access rule nodemap. Default is
    READ_WRITE.

    Values:
      ACCESS_MODE_UNSPECIFIED: Unspecified access mode.
      READ_ONLY: Read-only access mode.
      READ_WRITE: Read-write access mode.
    """
    ACCESS_MODE_UNSPECIFIED = 0
    READ_ONLY = 1
    READ_WRITE = 2

  class SquashModeValueValuesEnum(_messages.Enum):
    r"""Required. Squash mode for the access rule.

    Values:
      SQUASH_MODE_UNSPECIFIED: Unspecified squash mode.
      NO_SQUASH: Squash is disabled. If set inside an AccessRule, root users
        matching the ip_ranges are not squashed. If set as the
        default_squash_mode, root squash is disabled for this instance. If the
        default squash mode is `NO_SQUASH`, do not set the default_squash_uid
        or default_squash_gid, or an `invalid argument` error is returned.
      ROOT_SQUASH: Root user squash is enabled. Not supported inside an
        AccessRule. If set as the default_squash_mode, root users not matching
        any of the access_rules are squashed to the default_squash_uid and
        default_squash_gid.
      ALL_USERS_SQUASH: All users squashed to the squash_uid and squash_gid
        for the access rule. If this is for the default_squash_mode, then the
        default_squash_uid and default_squash_gid will be squashed.
    """
    SQUASH_MODE_UNSPECIFIED = 0
    NO_SQUASH = 1
    ROOT_SQUASH = 2
    ALL_USERS_SQUASH = 3

  accessMode = _messages.EnumField('AccessModeValueValuesEnum', 1)
  ipAddressRanges = _messages.StringField(2, repeated=True)
  mountableSubdirectories = _messages.StringField(3, repeated=True)
  name = _messages.StringField(4)
  squashGid = _messages.IntegerField(5, variant=_messages.Variant.INT32)
  squashMode = _messages.EnumField('SquashModeValueValuesEnum', 6)
  squashUid = _messages.IntegerField(7, variant=_messages.Variant.INT32)


class AccessRulesOptions(_messages.Message):
  r"""IP-based access rules for the Managed Lustre instance. These options
  define the root user squash configuration.

  Enums:
    DefaultSquashModeValueValuesEnum: Required. The squash mode for the
      default access rule.

  Fields:
    accessRules: Optional. The access rules for the instance.
    defaultMountableSubdirectories: Optional. The list of non-root directories
      that can be mounted from clients in the default access rule. Currently,
      there can be only a single directory at most. If no directory is
      mentioned, then the root directory will be accessible.
    defaultSquashGid: Optional. The user squash GID for the default access
      rule. This user squash GID applies to all root users connecting from
      clients that are not matched by any of the access rules. If not set, the
      default is 0 (no GID squash).
    defaultSquashMode: Required. The squash mode for the default access rule.
    defaultSquashUid: Optional. The user squash UID for the default access
      rule. This user squash UID applies to all root users connecting from
      clients that are not matched by any of the access rules. If not set, the
      default is 0 (no UID squash).
  """

  class DefaultSquashModeValueValuesEnum(_messages.Enum):
    r"""Required. The squash mode for the default access rule.

    Values:
      SQUASH_MODE_UNSPECIFIED: Unspecified squash mode.
      NO_SQUASH: Squash is disabled. If set inside an AccessRule, root users
        matching the ip_ranges are not squashed. If set as the
        default_squash_mode, root squash is disabled for this instance. If the
        default squash mode is `NO_SQUASH`, do not set the default_squash_uid
        or default_squash_gid, or an `invalid argument` error is returned.
      ROOT_SQUASH: Root user squash is enabled. Not supported inside an
        AccessRule. If set as the default_squash_mode, root users not matching
        any of the access_rules are squashed to the default_squash_uid and
        default_squash_gid.
      ALL_USERS_SQUASH: All users squashed to the squash_uid and squash_gid
        for the access rule. If this is for the default_squash_mode, then the
        default_squash_uid and default_squash_gid will be squashed.
    """
    SQUASH_MODE_UNSPECIFIED = 0
    NO_SQUASH = 1
    ROOT_SQUASH = 2
    ALL_USERS_SQUASH = 3

  accessRules = _messages.MessageField('AccessRule', 1, repeated=True)
  defaultMountableSubdirectories = _messages.StringField(2, repeated=True)
  defaultSquashGid = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  defaultSquashMode = _messages.EnumField('DefaultSquashModeValueValuesEnum', 4)
  defaultSquashUid = _messages.IntegerField(5, variant=_messages.Variant.INT32)


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Date(_messages.Message):
  r"""Represents a whole or partial calendar date, such as a birthday. The
  time of day and time zone are either specified elsewhere or are
  insignificant. The date is relative to the Gregorian Calendar. This can
  represent one of the following: * A full date, with non-zero year, month,
  and day values. * A month and day, with a zero year (for example, an
  anniversary). * A year on its own, with a zero month and a zero day. * A
  year and month, with a zero day (for example, a credit card expiration
  date). Related types: * google.type.TimeOfDay * google.type.DateTime *
  google.protobuf.Timestamp

  Fields:
    day: Day of a month. Must be from 1 to 31 and valid for the year and
      month, or 0 to specify a year by itself or a year and month where the
      day isn't significant.
    month: Month of a year. Must be from 1 to 12, or 0 to specify a year
      without a month and day.
    year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
      without a year.
  """

  day = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  month = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  year = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class DirectoryPolicy(_messages.Message):
  r"""A directory policy for a Managed Lustre instance.

  Fields:
    directoryPath: Required. Immutable. The lustre instance filesystem full
      path of the directory. It must start with a slash. e.g.
      /lustre/testFolder
    lustreProjectId: Output only. The lustre project ID assigned for the
      directory by the service. This read-only ID can be used to manage
      quotas. See more details in https://docs.cloud.google.com/managed-
      lustre/docs/file-system-quotas#set_quotas
    name: Immutable. Identifier. The resource name of the directory policy.
      DirectoryPolicy names have the form `projects/{project}/locations/{locat
      ion}/instances/{instance}/directoryPolicies/{id}`. {id} is user
      provided.
    uid: Output only. Unique ID of the resource.
  """

  directoryPath = _messages.StringField(1)
  lustreProjectId = _messages.IntegerField(2)
  name = _messages.StringField(3)
  uid = _messages.StringField(4)


class DynamicTierOptions(_messages.Message):
  r"""Dynamic tier options for a Managed Lustre instance.

  Enums:
    ModeValueValuesEnum: Required. Immutable. The dynamic tier mode of the
      instance.

  Fields:
    mode: Required. Immutable. The dynamic tier mode of the instance.
  """

  class ModeValueValuesEnum(_messages.Enum):
    r"""Required. Immutable. The dynamic tier mode of the instance.

    Values:
      MODE_UNSPECIFIED: Unspecified dynamic tier mode.
      DISABLED: The dynamic tier is explicitly disabled.
      DEFAULT_CACHE: The dynamic tier is enabled.
    """
    MODE_UNSPECIFIED = 0
    DISABLED = 1
    DEFAULT_CACHE = 2

  mode = _messages.EnumField('ModeValueValuesEnum', 1)


class ExportDataRequest(_messages.Message):
  r"""Export data from Managed Lustre to a Cloud Storage bucket.

  Fields:
    gcsPath: The URI to a Cloud Storage bucket, or a path within a bucket,
      using the format `gs:////`. If a path inside the bucket is specified, it
      must end with a forward slash (`/`).
    lustrePath: The root directory path to the Managed Lustre file system.
      Must start with `/`. Default is `/`.
    requestId: Optional. UUID to identify requests.
    serviceAccount: Optional. User-specified service account used to perform
      the transfer. If unspecified, the Managed Lustre service agent is used.
      Use one of the following formats: * `{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/{PROJECT_ID}/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/-/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}`
  """

  gcsPath = _messages.MessageField('GcsPath', 1)
  lustrePath = _messages.MessageField('LustrePath', 2)
  requestId = _messages.StringField(3)
  serviceAccount = _messages.StringField(4)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class GcsPath(_messages.Message):
  r"""Specifies a Cloud Storage bucket and, optionally, a path inside the
  bucket.

  Fields:
    uri: Required. The URI to a Cloud Storage bucket, or a path within a
      bucket, using the format `gs:////`. If a path inside the bucket is
      specified, it must end with a forward slash (`/`).
  """

  uri = _messages.StringField(1)


class GoogleProtobufEmpty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ImportDataRequest(_messages.Message):
  r"""Message for importing data to Lustre.

  Fields:
    gcsPath: The Cloud Storage source bucket and, optionally, path inside the
      bucket. If a path inside the bucket is specified, it must end with a
      forward slash (`/`).
    lustrePath: Lustre path destination.
    requestId: Optional. UUID to identify requests.
    serviceAccount: Optional. User-specified service account used to perform
      the transfer. If unspecified, the default Managed Lustre service agent
      will be used. Use one of the following formats: *
      `{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/{PROJECT_ID}/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/-/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}`
  """

  gcsPath = _messages.MessageField('GcsPath', 1)
  lustrePath = _messages.MessageField('LustrePath', 2)
  requestId = _messages.StringField(3)
  serviceAccount = _messages.StringField(4)


class Instance(_messages.Message):
  r"""A Managed Lustre instance.

  Enums:
    AuditLogLevelValueValuesEnum: Optional. The file system audit log level
      for the instance.
    InitialCompressionAlgorithmValueValuesEnum: Optional. Input only. The
      compression algorithm to use when initializing the instance.
    StateValueValuesEnum: Output only. The state of the instance.

  Messages:
    LabelsValue: Optional. Labels as key value pairs.

  Fields:
    accessRulesOptions: Optional. The access rules options for the instance.
    auditLogLevel: Optional. The file system audit log level for the instance.
    capacityGib: Required. The storage capacity of the instance in gibibytes
      (GiB). Allowed values depend on the `perUnitStorageThroughput`. See
      [Performance tiers](https://docs.cloud.google.com/managed-
      lustre/docs/performance-tiers) for specific minimums, maximums, and step
      sizes for each performance tier.
    createTime: Output only. Timestamp when the instance was created.
    description: Optional. A user-readable description of the instance.
    dynamicTierOptions: Optional. Immutable. Specifies whether the instance is
      on the Dynamic tier. See [Performance
      tiers](https://docs.cloud.google.com/managed-lustre/docs/performance-
      tiers) for more information.
    filesystem: Required. Immutable. The filesystem name for this instance.
      This name is used by client-side tools, including when mounting the
      instance. Must be eight characters or less and can only contain letters
      and numbers.
    gkeSupportEnabled: Optional. Deprecated: No longer required for GKE
      instance creation. Indicates whether you want to enable support for GKE
      clients. By default, GKE clients are not supported.
    iamAccessControlEnabled: Optional. Whether the IAM-based access control is
      enabled or not.
    initialCompressionAlgorithm: Optional. Input only. The compression
      algorithm to use when initializing the instance.
    initialCompressionLevel: Optional. Input only. The compression level to
      use for when initializing the instance. Defaults to 1 if not set. Valid
      ranges depend on the selected algorithm: - LZ4: 1 to 12 - ZSTD: 1 to 22
      Higher levels generally offer better compression ratios but may increase
      CPU usage.
    kmsKey: Optional. Immutable. The Cloud KMS key name to use for data
      encryption. If not set, the instance will use Google-managed encryption
      keys. If set, the instance will use customer-managed encryption keys.
      The key must be in the same region as the instance. The key format is: p
      rojects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{k
      ey}
    labels: Optional. Labels as key value pairs.
    maintenancePolicy: Optional. The maintenance policy for the instance to
      determine when to allow or exclude the instance from maintenance
      updates.
    mountPoint: Output only. Mount point of the instance in the format
      `IP_ADDRESS@tcp:/FILESYSTEM`.
    name: Identifier. The name of the instance.
    network: Required. Immutable. The full name of the VPC network to which
      the instance is connected. Must be in the format
      `projects/{project_id}/global/networks/{network_name}`.
    perUnitStorageThroughput: Optional. The throughput of the instance in MBps
      per TiB. Valid values are 0, 125, 250, 500, 1000. See [Performance
      tiers](https://docs.cloud.google.com/managed-lustre/docs/performance-
      tiers) for more information. If the instance is using the Dynamic tier,
      this field must not be set or must be set to zero.
    placementPolicy: Optional. The placement policy name for the instance in
      the format of projects/{project}/locations/{location}/resourcePolicies/{
      resource_policy}
    satisfiesPzi: Output only. Reserved for future use
    satisfiesPzs: Output only. Reserved for future use
    state: Output only. The state of the instance.
    stateReason: Output only. The reason why the instance is in a certain
      state (e.g. SUSPENDED).
    uid: Output only. Unique ID of the resource. This is unrelated to the
      access rules which allow specifying the root squash uid.
    upcomingMaintenanceSchedule: Output only. Date and time of upcoming
      maintenance for the instance, if a maintenance policy is set.
    updateTime: Output only. Timestamp when the instance was last updated.
  """

  class AuditLogLevelValueValuesEnum(_messages.Enum):
    r"""Optional. The file system audit log level for the instance.

    Values:
      AUDIT_LOG_LEVEL_UNSPECIFIED: Not set.
      DISABLED: File system audit logs are explicitly disabled.
      BASIC: File system basic audit logs are enabled.
    """
    AUDIT_LOG_LEVEL_UNSPECIFIED = 0
    DISABLED = 1
    BASIC = 2

  class InitialCompressionAlgorithmValueValuesEnum(_messages.Enum):
    r"""Optional. Input only. The compression algorithm to use when
    initializing the instance.

    Values:
      COMPRESSION_ALGORITHM_UNSPECIFIED: Not set.
      LZ4: Compression algorithm is explicitly enabled with LZ4 compression.
      ZSTD: Compression algorithm is explicitly enabled with ZSTD compression.
    """
    COMPRESSION_ALGORITHM_UNSPECIFIED = 0
    LZ4 = 1
    ZSTD = 2

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the instance.

    Values:
      STATE_UNSPECIFIED: Not set.
      ACTIVE: The instance is available for use.
      CREATING: The instance is being created and is not yet ready for use.
      DELETING: The instance is being deleted.
      UPGRADING: The instance is being upgraded.
      REPAIRING: The instance is being repaired.
      STOPPED: The instance is stopped.
      UPDATING: The instance is being updated.
      SUSPENDED: The instance is suspended due to an issue related to Cloud
        KMS. The details are available in state_reason.
      BACKING_UP: The instance is being backed up.
      MAINTENANCE: The instance is under maintenance.
    """
    STATE_UNSPECIFIED = 0
    ACTIVE = 1
    CREATING = 2
    DELETING = 3
    UPGRADING = 4
    REPAIRING = 5
    STOPPED = 6
    UPDATING = 7
    SUSPENDED = 8
    BACKING_UP = 9
    MAINTENANCE = 10

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  accessRulesOptions = _messages.MessageField('AccessRulesOptions', 1)
  auditLogLevel = _messages.EnumField('AuditLogLevelValueValuesEnum', 2)
  capacityGib = _messages.IntegerField(3)
  createTime = _messages.StringField(4)
  description = _messages.StringField(5)
  dynamicTierOptions = _messages.MessageField('DynamicTierOptions', 6)
  filesystem = _messages.StringField(7)
  gkeSupportEnabled = _messages.BooleanField(8)
  iamAccessControlEnabled = _messages.BooleanField(9)
  initialCompressionAlgorithm = _messages.EnumField('InitialCompressionAlgorithmValueValuesEnum', 10)
  initialCompressionLevel = _messages.IntegerField(11, variant=_messages.Variant.INT32)
  kmsKey = _messages.StringField(12)
  labels = _messages.MessageField('LabelsValue', 13)
  maintenancePolicy = _messages.MessageField('MaintenancePolicy', 14)
  mountPoint = _messages.StringField(15)
  name = _messages.StringField(16)
  network = _messages.StringField(17)
  perUnitStorageThroughput = _messages.IntegerField(18)
  placementPolicy = _messages.StringField(19)
  satisfiesPzi = _messages.BooleanField(20)
  satisfiesPzs = _messages.BooleanField(21)
  state = _messages.EnumField('StateValueValuesEnum', 22)
  stateReason = _messages.StringField(23)
  uid = _messages.StringField(24)
  upcomingMaintenanceSchedule = _messages.MessageField('MaintenanceSchedule', 25)
  updateTime = _messages.StringField(26)


class ListDirectoryPoliciesResponse(_messages.Message):
  r"""Response message for ListDirectoryPolicies.

  Fields:
    directoryPolicies: The list of DirectoryPolicies.
    nextPageToken: A token identifying a page of results the server should
      return.
  """

  directoryPolicies = _messages.MessageField('DirectoryPolicy', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListInstancesResponse(_messages.Message):
  r"""Message for response to listing Instances

  Fields:
    instances: Response from ListInstances.
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Unordered list. Locations that could not be reached.
  """

  instances = _messages.MessageField('Instance', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LustrePath(_messages.Message):
  r"""The root directory path to the Lustre file system.

  Fields:
    path: Optional. The root directory path to the Managed Lustre file system.
      Must start with `/`. Default is `/`. If you're importing data into
      Managed Lustre, any path other than the default must already exist on
      the file system.
  """

  path = _messages.StringField(1)


class LustreProjectsLocationsGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesCreateRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesCreateRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    instanceId: Required. The name of the Managed Lustre instance. * Must
      contain only lowercase letters, numbers, and hyphens. * Must start with
      a letter. * Must be between 1-63 characters. * Must end with a number or
      a letter.
    parent: Required. The instance's project and location, in the format
      `projects/{project}/locations/{location}`. Locations map to Google Cloud
      zones; for example, `us-west1-b`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  instance = _messages.MessageField('Instance', 1)
  instanceId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class LustreProjectsLocationsInstancesDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDeleteRequest object.

  Fields:
    force: Optional. If set to true, any sub-resources from this instance will
      also be deleted. Otherwise, the request will only work if the instance
      has no sub-resources.
    name: Required. The resource name of the instance to delete, in the format
      `projects/{projectId}/locations/{location}/instances/{instanceId}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest object.

  Fields:
    directoryPolicy: A DirectoryPolicy resource to be passed as the request
      body.
    directoryPolicyId: Required. The ID for the DirectoryPolicy to create.
    parent: Required. The parent instance. It must be in the format of
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  directoryPolicy = _messages.MessageField('DirectoryPolicy', 1)
  directoryPolicyId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest object.

  Fields:
    name: Required. The resource name of the directory policy. DirectoryPolicy
      names have the form `projects/{project}/locations/{location}/instances/{
      instance}/directoryPolicies/{id}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest
  object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest object.

  Fields:
    name: Required. The resource name of the directory policy. DirectoryPolicy
      names have the form `projects/{project}/locations/{location}/instances/{
      instance}/directoryPolicies/{id}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesListRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server might return fewer items
      than requested. If unspecified, the server will pick an appropriate
      default. The maximum value is 1000; values above 1000 will be coerced to
      1000.
    pageToken: Optional. A page token, received from a previous
      `ListDirectoryPolicies` call. Provide this to retrieve the subsequent
      page. When paginating, all other parameters provided to
      `ListDirectoryPolicies` must match the call that provided the page
      token.
    parent: Required. The parent instance. It must be in the format of
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest(_messages.Message):
  r"""A
  LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class LustreProjectsLocationsInstancesExportDataRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesExportDataRequest object.

  Fields:
    exportDataRequest: A ExportDataRequest resource to be passed as the
      request body.
    name: Required. The name of the Managed Lustre instance in the format
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  exportDataRequest = _messages.MessageField('ExportDataRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesGetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesGetIamPolicyRequest object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesGetRequest object.

  Fields:
    name: Required. The instance resource name, in the format
      `projects/{projectId}/locations/{location}/instances/{instanceId}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesImportDataRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesImportDataRequest object.

  Fields:
    importDataRequest: A ImportDataRequest resource to be passed as the
      request body.
    name: Required. The name of the Managed Lustre instance in the format
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  importDataRequest = _messages.MessageField('ImportDataRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesListRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Desired order of results.
    pageSize: Optional. Requested page size. Server might return fewer items
      than requested. If unspecified, the server will pick an appropriate
      default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The project and location for which to retrieve a list of
      instances, in the format `projects/{projectId}/locations/{location}`. To
      retrieve instance information for all locations, use "-" as the value of
      `{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class LustreProjectsLocationsInstancesPatchRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesPatchRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    name: Identifier. The name of the instance.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Specifies the fields to be overwritten in the
      instance resource by the update. The fields specified in the update_mask
      are relative to the resource, not the full request. A field will be
      overwritten if it is in the mask. If no mask is provided then all fields
      present in the request are overwritten.
  """

  instance = _messages.MessageField('Instance', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class LustreProjectsLocationsInstancesRescheduleMaintenanceRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesRescheduleMaintenanceRequest object.

  Fields:
    name: Required. Format:
      projects/{project}/locations/{location}/instances/{instance}
    rescheduleMaintenanceRequest: A RescheduleMaintenanceRequest resource to
      be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  rescheduleMaintenanceRequest = _messages.MessageField('RescheduleMaintenanceRequest', 2)


class LustreProjectsLocationsInstancesSetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesSetIamPolicyRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class LustreProjectsLocationsInstancesTestIamPermissionsRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesTestIamPermissionsRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class LustreProjectsLocationsListRequest(_messages.Message):
  r"""A LustreProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class LustreProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class MaintenanceExclusionWindow(_messages.Message):
  r"""Exclusion period when maintenance updates should not occur. An exclusion
  window can be in either of the following two formats: * Non-recurring : A
  full date, with non-zero year, month and day values. * Recurring : A month
  and day value, with a zero year. Time zone is UTC.

  Fields:
    endDate: Required. End date of the exclusion period in UTC time zone. This
      date is inclusive.
    startDate: Required. Start date of the exclusion period in UTC time zone.
      This date is inclusive.
    time: Required. Time in UTC when the exclusion window starts on start_date
      and ends on end_date. This can be: * Full time OR * All zeros for
      00:00:00 UTC
  """

  endDate = _messages.MessageField('Date', 1)
  startDate = _messages.MessageField('Date', 2)
  time = _messages.MessageField('TimeOfDay', 3)


class MaintenancePolicy(_messages.Message):
  r"""Defines a maintenance policy for a resource.

  Fields:
    maintenanceExclusionWindow: Optional. The exclusion windows for the
      instance. Currently limited to 1 window.
    weeklyMaintenanceWindows: Required. The weekly maintenance windows for the
      instance. Currently limited to 1 window.
  """

  maintenanceExclusionWindow = _messages.MessageField('MaintenanceExclusionWindow', 1, repeated=True)
  weeklyMaintenanceWindows = _messages.MessageField('WeeklyMaintenanceWindow', 2, repeated=True)


class MaintenanceSchedule(_messages.Message):
  r"""Represents a scheduled maintenance event.

  Fields:
    endTime: Output only. The scheduled end time for the maintenance.
    startTime: Output only. The scheduled start time for the maintenance.
  """

  endTime = _messages.StringField(1)
  startTime = _messages.StringField(2)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of a long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class ReconciliationOperationMetadata(_messages.Message):
  r"""Operation metadata returned by the CLH during resource state
  reconciliation.

  Enums:
    ExclusiveActionValueValuesEnum: Excluisive action returned by the CLH.

  Fields:
    deleteResource: DEPRECATED. Use exclusive_action instead.
    exclusiveAction: Excluisive action returned by the CLH.
  """

  class ExclusiveActionValueValuesEnum(_messages.Enum):
    r"""Excluisive action returned by the CLH.

    Values:
      UNKNOWN_REPAIR_ACTION: Unknown repair action.
      DELETE: The resource has to be deleted. When using this bit, the CLH
        should fail the operation. DEPRECATED. Instead use DELETE_RESOURCE
        OperationSignal in SideChannel.
      RETRY: This resource could not be repaired but the repair should be
        tried again at a later time. This can happen if there is a dependency
        that needs to be resolved first- e.g. if a parent resource must be
        repaired before a child resource.
    """
    UNKNOWN_REPAIR_ACTION = 0
    DELETE = 1
    RETRY = 2

  deleteResource = _messages.BooleanField(1)
  exclusiveAction = _messages.EnumField('ExclusiveActionValueValuesEnum', 2)


class Reschedule(_messages.Message):
  r"""The desired reschedule settings.

  Enums:
    RescheduleTypeValueValuesEnum: Required. The type of rescheduling.

  Fields:
    rescheduleType: Required. The type of rescheduling.
    scheduleTime: Optional. Required if reschedule_type is BY_TIME. Timestamp
      when the maintenance shall be rescheduled to. This time must be within
      28 days of the original scheduled maintenance start time.
  """

  class RescheduleTypeValueValuesEnum(_messages.Enum):
    r"""Required. The type of rescheduling.

    Values:
      RESCHEDULE_TYPE_UNSPECIFIED: Unspecified schedule type.
      IMMEDIATE: Apply update immediately
      NEXT_AVAILABLE_WINDOW: Reschedule to the next available window.
      BY_TIME: Reschedule to a specific time.
    """
    RESCHEDULE_TYPE_UNSPECIFIED = 0
    IMMEDIATE = 1
    NEXT_AVAILABLE_WINDOW = 2
    BY_TIME = 3

  rescheduleType = _messages.EnumField('RescheduleTypeValueValuesEnum', 1)
  scheduleTime = _messages.StringField(2)


class RescheduleMaintenanceRequest(_messages.Message):
  r"""Message for requesting to reschedule a maintenance event for a specific
  instance.

  Fields:
    requestId: Optional. A unique identifier for this request. A random UUID
      is recommended. This request is only idempotent if a `request_id` is
      provided.
    reschedule: Required. The desired reschedule settings.
  """

  requestId = _messages.StringField(1)
  reschedule = _messages.MessageField('Reschedule', 2)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


class TimeOfDay(_messages.Message):
  r"""Represents a time of day. The date and time zone are either not
  significant or are specified elsewhere. An API may choose to allow leap
  seconds. Related types are google.type.Date and `google.protobuf.Timestamp`.

  Fields:
    hours: Hours of a day in 24 hour format. Must be greater than or equal to
      0 and typically must be less than or equal to 23. An API may choose to
      allow the value "24:00:00" for scenarios like business closing time.
    minutes: Minutes of an hour. Must be greater than or equal to 0 and less
      than or equal to 59.
    nanos: Fractions of seconds, in nanoseconds. Must be greater than or equal
      to 0 and less than or equal to 999,999,999.
    seconds: Seconds of a minute. Must be greater than or equal to 0 and
      typically must be less than or equal to 59. An API may allow the value
      60 if it allows leap-seconds.
  """

  hours = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  minutes = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  nanos = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  seconds = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class WeeklyMaintenanceWindow(_messages.Message):
  r"""Weekly time window in which maintenance updates may occur. Duration of
  the window is currently fixed at 1 hour. Time zone is UTC.

  Enums:
    DayOfWeekValueValuesEnum: Required. Day of the week for the maintenance
      window.

  Fields:
    dayOfWeek: Required. Day of the week for the maintenance window.
    startTime: Required. Start time of the maintenance window in UTC time
      zone.
  """

  class DayOfWeekValueValuesEnum(_messages.Enum):
    r"""Required. Day of the week for the maintenance window.

    Values:
      DAY_OF_WEEK_UNSPECIFIED: The day of the week is unspecified.
      MONDAY: Monday
      TUESDAY: Tuesday
      WEDNESDAY: Wednesday
      THURSDAY: Thursday
      FRIDAY: Friday
      SATURDAY: Saturday
      SUNDAY: Sunday
    """
    DAY_OF_WEEK_UNSPECIFIED = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7

  dayOfWeek = _messages.EnumField('DayOfWeekValueValuesEnum', 1)
  startTime = _messages.MessageField('TimeOfDay', 2)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    LustreProjectsLocationsInstancesGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')
encoding.AddCustomJsonFieldMapping(
    LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')
