# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags module for Data Lineage commands."""

import argparse

ArgumentParser = argparse.ArgumentParser


def AddParentArgs(parser: ArgumentParser) -> None:
  """Adds mutually exclusive args for the parent resource to the parser."""
  parent_group = parser.add_mutually_exclusive_group(required=False)
  parent_group.add_argument(
      '--project',
      help=(
          'Project ID or number. If none of --project, --folder, or '
          '--organization are provided, the current project will be used.'
      ),
  )
  parent_group.add_argument('--folder', help='Folder ID.')
  parent_group.add_argument('--organization', help='Organization ID.')
