# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating snapshot groups."""


from googlecloudsdk.calliope import base

DETAILED_HELP = {  # Dict[str, str]
    'brief': 'Create, list, describe and delete Compute Engine snapshot groups',
}


# --- Alpha Group Definition ---
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class SnapshotGroupsAlpha(base.Group):
  """Create, list, describe and delete Compute Engine snapshot groups."""

  category = base.INSTANCES_CATEGORY
  detailed_help = DETAILED_HELP.copy()


# --- Beta Group Definition ---
@base.Hidden  # This hides the BETA group from public documentation
@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.UniverseCompatible
class SnapshotGroupsBeta(base.Group):
  """Create, list, describe and delete Compute Engine snapshot groups."""

  category = base.INSTANCES_CATEGORY
  detailed_help = DETAILED_HELP.copy()
