"""Generated client library for agentidentity version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.agentidentity.v1alpha import agentidentity_v1alpha_messages as messages


class AgentidentityV1alpha(base_api.BaseApiClient):
  """Generated client library for service agentidentity version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://agentidentity.googleapis.com/'
  MTLS_BASE_URL = 'https://agentidentity.mtls.googleapis.com/'

  _PACKAGE = 'agentidentity'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'AgentidentityV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new agentidentity handle."""
    url = url or self.BASE_URL
    super(AgentidentityV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_accessSummaries = self.ProjectsLocationsAccessSummariesService(self)
    self.projects_locations_authProviders_authorizations = self.ProjectsLocationsAuthProvidersAuthorizationsService(self)
    self.projects_locations_authProviders = self.ProjectsLocationsAuthProvidersService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsAccessSummariesService(base_api.BaseApiService):
    """Service class for the projects_locations_accessSummaries resource."""

    _NAME = 'projects_locations_accessSummaries'

    def __init__(self, client):
      super(AgentidentityV1alpha.ProjectsLocationsAccessSummariesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single AccessSummary.

      Args:
        request: (AgentidentityProjectsLocationsAccessSummariesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AccessSummary) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/accessSummaries/{accessSummariesId}',
        http_method='GET',
        method_id='agentidentity.projects.locations.accessSummaries.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAccessSummariesGetRequest',
        response_type_name='AccessSummary',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists AccessSummaries in a given project and location. Supported Filters: - `workload_id`: Filter by the SPIFFE ID of the agent. Example: `workload_id="spiffe://example.com/ns/default/sa/my-agent"`.

      Args:
        request: (AgentidentityProjectsLocationsAccessSummariesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAccessSummariesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/accessSummaries',
        http_method='GET',
        method_id='agentidentity.projects.locations.accessSummaries.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/accessSummaries',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAccessSummariesListRequest',
        response_type_name='ListAccessSummariesResponse',
        supports_download=False,
    )

  class ProjectsLocationsAuthProvidersAuthorizationsService(base_api.BaseApiService):
    """Service class for the projects_locations_authProviders_authorizations resource."""

    _NAME = 'projects_locations_authProviders_authorizations'

    def __init__(self, client):
      super(AgentidentityV1alpha.ProjectsLocationsAuthProvidersAuthorizationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes a single Authorization.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersAuthorizationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}/authorizations/{authorizationsId}',
        http_method='DELETE',
        method_id='agentidentity.projects.locations.authProviders.authorizations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersAuthorizationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Authorization.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersAuthorizationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Authorization) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}/authorizations/{authorizationsId}',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.authorizations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersAuthorizationsGetRequest',
        response_type_name='Authorization',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Authorizations in a given project and location.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersAuthorizationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAuthorizationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}/authorizations',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.authorizations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/authorizations',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersAuthorizationsListRequest',
        response_type_name='ListAuthorizationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsAuthProvidersService(base_api.BaseApiService):
    """Service class for the projects_locations_authProviders resource."""

    _NAME = 'projects_locations_authProviders'

    def __init__(self, client):
      super(AgentidentityV1alpha.ProjectsLocationsAuthProvidersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new AuthProvider in a given project and location.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['authProviderId', 'requestId'],
        relative_path='v1alpha/{+parent}/authProviders',
        request_field='authProvider',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersCreateRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}',
        http_method='DELETE',
        method_id='agentidentity.projects.locations.authProviders.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Disable(self, request, global_params=None):
      r"""Disables a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersDisableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Disable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Disable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:disable',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.disable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:disable',
        request_field='disableAuthProviderRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersDisableRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

    def Enable(self, request, global_params=None):
      r"""Enables a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersEnableRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Enable')
      return self._RunMethod(
          config, request, global_params=global_params)

    Enable.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:enable',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.enable',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:enable',
        request_field='enableAuthProviderRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersEnableRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersGetRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:getIamPolicy',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1alpha/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists AuthProviders in a given project and location.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAuthProvidersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken', 'showDeleted'],
        relative_path='v1alpha/{+parent}/authProviders',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersListRequest',
        response_type_name='ListAuthProvidersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}',
        http_method='PATCH',
        method_id='agentidentity.projects.locations.authProviders.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='authProvider',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersPatchRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

    def Query(self, request, global_params=None):
      r"""Queries what all auth_providers are used by a given workload_id.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersQueryRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QueryAuthProvidersResponse) The response message.
      """
      config = self.GetMethodConfig('Query')
      return self._RunMethod(
          config, request, global_params=global_params)

    Query.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders:query',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.query',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken', 'workloadId'],
        relative_path='v1alpha/{+parent}/authProviders:query',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersQueryRequest',
        response_type_name='QueryAuthProvidersResponse',
        supports_download=False,
    )

    def QueryWorkloads(self, request, global_params=None):
      r"""Queries what all workloads are using a given auth_provider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersQueryWorkloadsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QueryWorkloadsResponse) The response message.
      """
      config = self.GetMethodConfig('QueryWorkloads')
      return self._RunMethod(
          config, request, global_params=global_params)

    QueryWorkloads.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:queryWorkloads',
        http_method='GET',
        method_id='agentidentity.projects.locations.authProviders.queryWorkloads',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}:queryWorkloads',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersQueryWorkloadsRequest',
        response_type_name='QueryWorkloadsResponse',
        supports_download=False,
    )

    def RevokeAuthorization(self, request, global_params=None):
      r"""Revokes all authorizations for a specific user on an AuthProvider. This deletes all authorization records associated with the user and AuthProvider, effectively revoking access across all agents.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersRevokeAuthorizationRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RevokeAuthorizationResponse) The response message.
      """
      config = self.GetMethodConfig('RevokeAuthorization')
      return self._RunMethod(
          config, request, global_params=global_params)

    RevokeAuthorization.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:revokeAuthorization',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.revokeAuthorization',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:revokeAuthorization',
        request_field='revokeAuthorizationRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersRevokeAuthorizationRequest',
        response_type_name='RevokeAuthorizationResponse',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:setIamPolicy',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:testIamPermissions',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1alpha/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

    def Undelete(self, request, global_params=None):
      r"""Undeletes a single AuthProvider.

      Args:
        request: (AgentidentityProjectsLocationsAuthProvidersUndeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AuthProvider) The response message.
      """
      config = self.GetMethodConfig('Undelete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Undelete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/authProviders/{authProvidersId}:undelete',
        http_method='POST',
        method_id='agentidentity.projects.locations.authProviders.undelete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:undelete',
        request_field='undeleteAuthProviderRequest',
        request_type_name='AgentidentityProjectsLocationsAuthProvidersUndeleteRequest',
        response_type_name='AuthProvider',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(AgentidentityV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (AgentidentityProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='agentidentity.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (AgentidentityProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='agentidentity.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='AgentidentityProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(AgentidentityV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }
