# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update the configuration of an insight config."""

import datetime

from googlecloudsdk.api_lib.developer_connect.insights_configs import insights_config
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.developer_connect import flags
from googlecloudsdk.command_lib.developer_connect import resource_args
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION': """
          Create an insights config.
          """,
    'EXAMPLES': """
          To create an insights config with an apphub application, run:

            $ {command} insights-config-name --app-hub-application=projects/my-project/locations/us-central1/applications/my-app-hub-application

          To create an insights config with projects, run:

            $ {command} insights-config-name --target-projects=project1,project2
          """,
}


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Create an insight config."""

  @classmethod
  def Args(cls, parser):
    """Adds arguments for this command."""
    try:
      resource_args.AddInsightConfigResourceArg(parser, verb='create')
    except exceptions.HttpException as e:
      log.status.Print('Failed to add insight config resource argument.')
      raise e

    # Use cls.ReleaseTrack() to check if we are in Alpha
    is_alpha = cls.ReleaseTrack() == base.ReleaseTrack.ALPHA

    source_group = parser.add_mutually_exclusive_group(required=not is_alpha)
    flags.AddAppHubApplicationArgument(source_group)
    flags.AddTargetProjectsArgument(source_group)

    flags.AddArtifactConfigsArgument(parser)
    flags.AddSourceConfigArgument(parser)

  def Run(self, args):
    max_wait = datetime.timedelta(seconds=30)
    client = insights_config.InsightsConfigClient(self.ReleaseTrack())
    insights_config_ref = args.CONCEPTS.insights_config.Parse()
    try:
      operation = client.create(
          insight_config_ref=insights_config_ref,
          app_hub=args.app_hub_application,
          target_projects=args.target_projects,
          user_artifact_configs=args.artifact_configs,
          source_config=args.source_config,
      )
    except exceptions.HttpException as e:
      log.status.Print('Failed to create the insight config {}.'.format(
          insights_config_ref.RelativeName()
      ))
      raise e

    log.status.Print('Creating the insight config {}.'.format(
        insights_config_ref.RelativeName()
    ))

    return client.wait_for_operation(
        operation_ref=client.get_operation_ref(operation),
        message='Waiting for operation [{}] to be completed...'
        .format(
            client.get_operation_ref(operation).RelativeName()),
        has_result=True,
        max_wait=max_wait,
    )
Create.detailed_help = DETAILED_HELP
