"""Generated client library for datalineage version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.datalineage.v1 import datalineage_v1_messages as messages


class DatalineageV1(base_api.BaseApiClient):
  """Generated client library for service datalineage version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://datalineage.googleapis.com/'
  MTLS_BASE_URL = 'https://datalineage.mtls.googleapis.com/'

  _PACKAGE = 'datalineage'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform', 'https://www.googleapis.com/auth/datalineage.read-write', 'https://www.googleapis.com/auth/datalineage.readonly']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'DatalineageV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new datalineage handle."""
    url = url or self.BASE_URL
    super(DatalineageV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_locations_config = self.FoldersLocationsConfigService(self)
    self.folders_locations = self.FoldersLocationsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_locations_config = self.OrganizationsLocationsConfigService(self)
    self.organizations_locations = self.OrganizationsLocationsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_locations_config = self.ProjectsLocationsConfigService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_processes_runs_lineageEvents = self.ProjectsLocationsProcessesRunsLineageEventsService(self)
    self.projects_locations_processes_runs = self.ProjectsLocationsProcessesRunsService(self)
    self.projects_locations_processes = self.ProjectsLocationsProcessesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersLocationsConfigService(base_api.BaseApiService):
    """Service class for the folders_locations_config resource."""

    _NAME = 'folders_locations_config'

    def __init__(self, client):
      super(DatalineageV1.FoldersLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageFoldersLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/folders/{foldersId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.folders.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageFoldersLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/folders/{foldersId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.folders.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class FoldersLocationsService(base_api.BaseApiService):
    """Service class for the folders_locations resource."""

    _NAME = 'folders_locations'

    def __init__(self, client):
      super(DatalineageV1.FoldersLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(DatalineageV1.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsConfigService(base_api.BaseApiService):
    """Service class for the organizations_locations_config resource."""

    _NAME = 'organizations_locations_config'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageOrganizationsLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.organizations.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageOrganizationsLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.organizations.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class OrganizationsLocationsService(base_api.BaseApiService):
    """Service class for the organizations_locations resource."""

    _NAME = 'organizations_locations'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsConfigService(base_api.BaseApiService):
    """Service class for the projects_locations_config resource."""

    _NAME = 'projects_locations_config'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageProjectsLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.projects.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.projects.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='datalineage.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='googleLongrunningCancelOperationRequest',
        request_type_name='DatalineageProjectsLocationsOperationsCancelRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (DatalineageProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsGetRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='datalineage.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsListRequest',
        response_type_name='GoogleLongrunningListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProcessesRunsLineageEventsService(base_api.BaseApiService):
    """Service class for the projects_locations_processes_runs_lineageEvents resource."""

    _NAME = 'projects_locations_processes_runs_lineageEvents'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesRunsLineageEventsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new lineage event.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1LineageEvent) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/lineageEvents',
        request_field='googleCloudDatacatalogLineageV1LineageEvent',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1LineageEvent',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the lineage event with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents/{lineageEventsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a specified lineage event.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1LineageEvent) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents/{lineageEventsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1LineageEvent',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists lineage events in the given project and location. The list order is not defined.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListLineageEventsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/lineageEvents',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListLineageEventsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProcessesRunsService(base_api.BaseApiService):
    """Service class for the projects_locations_processes_runs resource."""

    _NAME = 'projects_locations_processes_runs'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesRunsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.runs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/runs',
        request_field='googleCloudDatacatalogLineageV1Run',
        request_type_name='DatalineageProjectsLocationsProcessesRunsCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the run with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.runs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of the specified run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists runs in the given project and location. List order is descending by `start_time`.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListRunsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/runs',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListRunsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='PATCH',
        method_id='datalineage.projects.locations.processes.runs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudDatacatalogLineageV1Run',
        request_type_name='DatalineageProjectsLocationsProcessesRunsPatchRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

  class ProjectsLocationsProcessesService(base_api.BaseApiService):
    """Service class for the projects_locations_processes resource."""

    _NAME = 'projects_locations_processes'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new process.

      Args:
        request: (DatalineageProjectsLocationsProcessesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/processes',
        request_field='googleCloudDatacatalogLineageV1Process',
        request_type_name='DatalineageProjectsLocationsProcessesCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the process with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of the specified process.

      Args:
        request: (DatalineageProjectsLocationsProcessesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List processes in the given project and location. List order is descending by insertion time.

      Args:
        request: (DatalineageProjectsLocationsProcessesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListProcessesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/processes',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListProcessesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a process.

      Args:
        request: (DatalineageProjectsLocationsProcessesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='PATCH',
        method_id='datalineage.projects.locations.processes.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'requestId', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudDatacatalogLineageV1Process',
        request_type_name='DatalineageProjectsLocationsProcessesPatchRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def BatchSearchLinkProcesses(self, request, global_params=None):
      r"""Retrieve information about LineageProcesses associated with specific links. LineageProcesses are transformation pipelines that result in data flowing from **source** to **target** assets. Links between assets represent this operation. If you have specific link names, you can use this method to verify which LineageProcesses contribute to creating those links. See the SearchLinks method for more information on how to retrieve link name. You can retrieve the LineageProcess information in every project where you have the `datalineage.events.get` permission. The project provided in the URL is used for Billing and Quota.

      Args:
        request: (DatalineageProjectsLocationsBatchSearchLinkProcessesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1BatchSearchLinkProcessesResponse) The response message.
      """
      config = self.GetMethodConfig('BatchSearchLinkProcesses')
      return self._RunMethod(
          config, request, global_params=global_params)

    BatchSearchLinkProcesses.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:batchSearchLinkProcesses',
        http_method='POST',
        method_id='datalineage.projects.locations.batchSearchLinkProcesses',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:batchSearchLinkProcesses',
        request_field='googleCloudDatacatalogLineageV1BatchSearchLinkProcessesRequest',
        request_type_name='DatalineageProjectsLocationsBatchSearchLinkProcessesRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1BatchSearchLinkProcessesResponse',
        supports_download=False,
    )

    def SearchLineageStreaming(self, request, global_params=None):
      r"""Retrieves a streaming response of lineage links connected to the requested assets by performing a breadth-first search in the given direction. Links represent the data flow between **source** (upstream) and **target** (downstream) assets in transformation pipelines. Links are stored in the same project as the Lineage Events that create them. This method retrieves links from all valid locations provided in the request. This method supports Column-Level Lineage (CLL) along with wildcard support to retrieve all CLL for an Entity FQN. Following permissions are required to retrieve links: * `datalineage.events.get` permission for the project where the link is stored for entity-level lineage. * `datalineage.events.getFields` permission for the project where the link is stored for column-level lineage. This method also returns processes that created the links if explicitly requested by setting [max_process_per_link](google.cloud.datacatalog.lineage.v1.SearchLineageStreamingRequest.limits.max_process_per_link) is non-zero and full process details are requested via `links.processes.process` in the [FieldMask](https://developers.google.com/workspace/docs/api/how-tos/field-masks#read_with_a_field_mask). Permission required to retrieve processes: * `datalineage.processes.get` permission for the project where the process is stored.

      Args:
        request: (DatalineageProjectsLocationsSearchLineageStreamingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1SearchLineageStreamingResponse) The response message.
      """
      config = self.GetMethodConfig('SearchLineageStreaming')
      return self._RunMethod(
          config, request, global_params=global_params)

    SearchLineageStreaming.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:searchLineageStreaming',
        http_method='POST',
        method_id='datalineage.projects.locations.searchLineageStreaming',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:searchLineageStreaming',
        request_field='googleCloudDatacatalogLineageV1SearchLineageStreamingRequest',
        request_type_name='DatalineageProjectsLocationsSearchLineageStreamingRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1SearchLineageStreamingResponse',
        supports_download=False,
    )

    def SearchLinks(self, request, global_params=None):
      r"""Retrieve a list of links connected to a specific asset. Links represent the data flow between **source** (upstream) and **target** (downstream) assets in transformation pipelines. Links are stored in the same project as the Lineage Events that create them. You can retrieve links in every project where you have the `datalineage.events.get` permission. The project provided in the URL is used for Billing and Quota.

      Args:
        request: (DatalineageProjectsLocationsSearchLinksRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1SearchLinksResponse) The response message.
      """
      config = self.GetMethodConfig('SearchLinks')
      return self._RunMethod(
          config, request, global_params=global_params)

    SearchLinks.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:searchLinks',
        http_method='POST',
        method_id='datalineage.projects.locations.searchLinks',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:searchLinks',
        request_field='googleCloudDatacatalogLineageV1SearchLinksRequest',
        request_type_name='DatalineageProjectsLocationsSearchLinksRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1SearchLinksResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(DatalineageV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }
