# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe a Device Run session."""

from googlecloudsdk.api_lib import device_run
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.device_run import resource_args


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Describe(base.DescribeCommand):
  """Describe a Device Run session."""

  @staticmethod
  def Args(parser):
    resource_args.AddSessionResourceArg(parser, 'describe')

  def Run(self, args):
    session_ref = args.CONCEPTS.session.Parse()
    client = device_run.SessionsClient(api_version='v1alpha')
    return client.Get(session_ref)


Describe.detailed_help = {
    'DESCRIPTION':
        'Describe a Device Run session.',
    'EXAMPLES':
        """\
To describe a session named `my-session` in location `us-central1`, run:

  $ {command} my-session --location=us-central1
""",
}
