# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for parsing config files."""


from googlecloudsdk.core import exceptions
from googlecloudsdk.core import yaml


def ReadAutokeyConfigFromConfigFile(file_path):
  """Fetches the AutokeyConfig from the config file."""
  try:
    parsed_yaml = yaml.load_path(file_path)
    etag = parsed_yaml['etag'] if 'etag' in parsed_yaml else ''
  except yaml.Error as error:
    raise exceptions.Error('unable to load kubeconfig for {0}: {1}'.format(
        file_path, error))
  if 'name' not in parsed_yaml:
    raise exceptions.Error('AutokeyConfig file must contain a name.')
  if 'keyProject' not in parsed_yaml:
    raise exceptions.Error('AutokeyConfig file must contain a keyProject.')
  return parsed_yaml['name'], parsed_yaml['keyProject'], etag


def ReadAutokeyConfigFromConfigFileWithKeyProjectResolutionMode(file_path: str):
  """Fetches the AutokeyConfig from the config file.

  Args:
    file_path: (str) The path to the YAML config file.

  Returns:
    A tuple (name, key_project_resolution_mode, key_project, etag), where name
    is the resource name of the AutokeyConfig, key_project_resolution_mode is
    the keyProjectResolutionMode field if present, key_project is the keyProject
    field value if present (or an empty string if set to null), otherwise None,
    and etag is the etag field if present, otherwise an empty string.

  Raises:
    googlecloudsdk.core.exceptions.Error: If the file cannot be loaded,
      is missing required fields ('name'), or has conflicting fields.
  """
  try:
    parsed_yaml = yaml.load_path(file_path)
  except yaml.Error as error:
    raise exceptions.Error(
        f'unable to load AutokeyConfig for {file_path}: {error}'
    ) from error

  name = parsed_yaml.get('name')
  if not name:
    raise exceptions.Error('AutokeyConfig file must contain a name.')
  if not name.startswith(('folders/', 'projects/')):
    raise exceptions.Error(
        'AutokeyConfig name must start with folders/ or projects/.'
    )

  key_project = None
  if 'keyProject' in parsed_yaml:
    key_project = parsed_yaml.get('keyProject') or ''

  return (
      name,
      parsed_yaml.get('keyProjectResolutionMode', ''),
      key_project,
      parsed_yaml.get('etag', ''),
  )
